//Microchip Solutions v2012-04-03
/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

******************************************************************************/

/******************************************************************************
* Filename:        main.c
* Dependancies:    USB Host Driver with Generic Client Driver
* Processor:       PIC24F256GB1xx
* Hardware:        Explorer 16 with USB PICtail Plus
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the CompanyE for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/

#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_generic.h"
#include "user.h"
#include "timer.h"

void Write_ACL_Command(char *, int);
void Write_HCI_Command(char *, int);

void hci_acl_init(void);
void l2cap_sdp_hid(void);
void l2cap(void);
void sdp(void);
void pre_hid(void);
void hid(void);

//for storing a BD_address of PC (6 bytes)
#define PROG_ADDR1 0xA700
#define PROG_ADDR2 0xA702
int addrOffset;

//#define DEBUG_MODE
// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

        _CONFIG1(WDTPS_PS1 & FWPSA_PR32 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
        _CONFIG2(POSCMOD_HS & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_NODIV & IESO_ON)
        _CONFIG3(WPFP_WPFP0 & SOSCSEL_IO & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
        _CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)

// *****************************************************************************

// Application States
typedef enum
{
    BT_INITIALIZE = 0,	// Initialize the app when a device is attached
    BT_STATE_IDLE,
    BT_STATE_PROCESS,
    BT_STATE_WAIT_DATA
} BT_STATE;

// Hci States
typedef enum
{
	HCI_CMD_RESET = 0,	// Initialize the hci when a device is attached
	HCI_CMD_READ_BD_ADDR,
	HCI_CMD_LOCAL_NAME,
	HCI_CMD_CLASS_DEVICE,
	HCI_CMD_SCAN_ENABLE,
	HCI_CMD_WAIT_CONNECTION,
	HCI_CMD_CONNECTION_ACCEPT,
	HCI_CMD_CREATE_CONNECTION,
	HCI_CMD_DISCONNECT_REQ,
	L2CAP_CON_REQ_TO,
	HCI_END,
} HCI_STATE;

// Hid States
typedef enum
{
	HID_START = 0,	// Initialize the hid when a device is attached
//	HID_WRITE
} HID_STATE;

// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************

BYTE	deviceAddress;	// Address of the device on the USB
BT_STATE  DemoState;	// Current state of the demo application
HCI_STATE  HciState;	// Current state of the demo application
HID_STATE HidState;
WORD data_size;

int data_num;
//unsigned char local_bd_addr[6];//PIC
//unsigned char remote_bd_addr[6];//PC
unsigned char buf[64];
unsigned char buf1[64];
unsigned char handle[2];
unsigned char src_cid[2];
unsigned char src_cid1[2];
unsigned char dst_cid[2];
unsigned char dst_cid1[2];

int pdata;
int step;
char message[30];

// flags
unsigned char l2cap_pass=0;
unsigned char l2cap_flag=0;
unsigned char sdp_pass=0;
unsigned char sdp_flag=0;
unsigned char hid_pass=0;
unsigned char hid_flag=0;
unsigned char ConnectSwitch;
unsigned char hci_busy_flag=0;
unsigned char acl_busy_flag=0;
unsigned char disconnection_flag=0;

// data for keyboard
unsigned char key_data[4];

// data for Mouse
BYTE movement_length;
BYTE vector = 0;
	//The direction that the mouse will move in
ROM signed char dir_table[]={-4,-4,-4, 0, 4, 4, 4, 0};
unsigned char mouse[3];

int count;

// data for SDP
unsigned char att1a[5][31]=
{
{0x26,0x20,0x1b,0x00,0x80,0x00,0xFF,0xFF,0x07,0xFF,0xFF,0x00,0x7b,0x00,0x76,0x36,0x01,0x00,0x36,0x00,0xfd,0x09,0x00,0x00,0x0a,0x00,0x01,0x00,0x00,0x09,0x00},
{0x26,0x10,0x1b,0x00,0x01,0x35,0x03,0x19,0x11,0x24,0x09,0x00,0x04,0x35,0x0d,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x11,0x35,0x03,0x19,0x00,0x11,0x09,0x00,0x05},
{0x26,0x10,0x1b,0x00,0x35,0x03,0x19,0x10,0x02,0x09,0x00,0x06,0x35,0x09,0x09,0x65,0x6e,0x09,0x00,0x6a,0x09,0x01,0x00,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19},
{0x26,0x10,0x1b,0x00,0x11,0x24,0x09,0x01,0x00,0x09,0x00,0x0d,0x35,0x0f,0x35,0x0d,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x13,0x35,0x03,0x19,0x00,0x11,0x09,0x01},
{0x26,0x10,0x18,0x00,0x00,0x25,0x03,0x6b,0x65,0x79,0x09,0x02,0x00,0x09,0x01,0x00,0x09,0x02,0x01,0x09,0x01,0x11,0x09,0x02,0x02,0x02,0x00,0x76,0xff,0xff,0xff}
};
unsigned char att1b[5][31]=
{
{0x26,0x20,0x1b,0x00,0x80,0x00,0xFF,0xFF,0x07,0xFF,0xFF,0x00,0x7b,0x00,0x76,0x08,0x04,0x09,0x02,0x03,0x08,0x33,0x09,0x02,0x04,0x28,0x00,0x09,0x02,0x05,0x28},
{0x26,0x10,0x1b,0x00,0x01,0x09,0x02,0x06,0x35,0x45,0x35,0x43,0x08,0x22,0x25,0x3f,0x05,0x01,0x09,0x06,0xa1,0x01,0x05,0x07,0x19,0xe0,0x29,0xe7,0x15,0x00,0x25},
{0x26,0x10,0x1b,0x00,0x01,0x75,0x01,0x95,0x08,0x81,0x02,0x95,0x01,0x75,0x08,0x81,0x01,0x95,0x05,0x75,0x01,0x05,0x08,0x19,0x01,0x29,0x05,0x91,0x02,0x95,0x01},
{0x26,0x10,0x1b,0x00,0x75,0x03,0x91,0x01,0x95,0x06,0x75,0x08,0x15,0x00,0x25,0x65,0x05,0x07,0x19,0x00,0x29,0x65,0x81,0x00,0xc0,0x09,0x02,0x07,0x35,0x08,0x35},
{0x26,0x10,0x18,0x00,0x06,0x09,0x04,0x09,0x09,0x01,0x00,0x09,0x02,0x08,0x28,0x00,0x09,0x02,0x09,0x28,0x01,0x09,0x02,0x0a,0x28,0x02,0x00,0xec,0xff,0xff,0xff}
};
unsigned char att1c[2][31]=
{
{0x26,0x20,0x1b,0x00,0x1f,0x00,0xFF,0xFF,0x07,0xFF,0xFF,0x00,0x1a,0x00,0x17,0x01,0x09,0x02,0x0b,0x09,0x01,0x00,0x09,0x02,0x0c,0x09,0x0c,0x80,0x09,0x02,0x0d},
{0x26,0x10,0x08,0x00,0x28,0x00,0x09,0x02,0x0e,0x28,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff}
};
unsigned char att2[4][31]=
{
{0x26,0x20,0x1b,0x00,0x68,0x00,0xFF,0xFF,0x07,0xFF,0xFF,0x00,0x63,0x00,0x60,0x36,0x00,0x5d,0x36,0x00,0x5a,0x09,0x00,0x00,0x0a,0x00,0x01,0x00,0x01,0x09,0x00},
{0x26,0x10,0x1b,0x00,0x01,0x35,0x03,0x19,0x12,0x00,0x09,0x00,0x04,0x35,0x0d,0x35,0x06,0x19,0x01,0x00,0x09,0x00,0x01,0x35,0x03,0x19,0x00,0x01,0x09,0x00,0x05},
{0x26,0x10,0x1b,0x00,0x35,0x03,0x19,0x10,0x02,0x09,0x00,0x09,0x35,0x08,0x35,0x06,0x19,0x12,0x00,0x09,0x01,0x00,0x09,0x02,0x00,0x09,0x01,0x00,0x09,0x02,0x01},
{0x26,0x10,0x1b,0x00,0x09,0x05,0x7e,0x09,0x02,0x02,0x09,0x03,0x09,0x09,0x02,0x03,0x09,0x06,0x00,0x09,0x02,0x04,0x28,0x01,0x09,0x02,0x05,0x09,0x00,0x02,0x00}
};


//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/

BOOL InitializeSystem ( void )
{

	unsigned int pll_startup_counter = 600;
	CLKDIVbits.PLLEN = 1;
	while(pll_startup_counter--);


	// Configure U2RX - put on pin 17 (RP8)
	RPINR19bits.U2RXR = 8;
	// Configure U2TX - put on pin 16 (RP7)
	RPOR3bits.RP7R = 5;
    // Init UART
    UART2Init();

	AD1PCFG = 0xFFFF; /*set AN pins to digital pin */
	TRISA=0x0003;	/* set PORTB(RA0 and RA1 pins) to input*/
					/* "0" corresponds to "o"utput and "1" to "i"nput */
	TRISB=0x001C;	/* set PORTB(RB2 and RB4 pins) to input*/
					/* "0" corresponds to "o"utput and "1" to "i"nput */
					/* SOSCSEL_IO setting in CONFIG3 to use RA4 and RB4 pins as digital ones */ 
	CNPU1=0x00CE;	/* Pull up CN1, 2, 3, 6, 7, pins  */

    // Set Default demo state
    DemoState = BT_INITIALIZE;
	HciState=HCI_CMD_RESET;
	HidState=HID_START;

	ConnectSwitch=PORTBbits.RB4;

    return TRUE;
} // InitializeSystem


/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         TRUE if a new device has been attached.  FALSE,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

BOOL CheckForNewAttach ( void )
{
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0)
    {
        GENERIC_DEVICE_ID DevID;

        DevID.vid   = 0x04D8;
        DevID.pid   = 0x000C;
        #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
            DevID.serialNumberLength = 0;
            DevID.serialNumber = NULL;
        #endif

        if (USBHostGenericGetDeviceAddress(&DevID))
        {
            deviceAddress = DevID.deviceAddress;
            UART2PrintString( "Generic demo device attached - polled, deviceAddress=" );
            UART2PutDec( deviceAddress );
            UART2PrintString( "\r\n" );
            return TRUE;
        }
    }

    return FALSE;

} // CheckForNewAttach


//********************************************************************************
//********************************************************************************
// Initialization for HCI and ACL
//********************************************************************************
//********************************************************************************
void hci_acl_init(void){
	switch (HciState)
	{
		case HCI_CMD_RESET:
		case HCI_CMD_READ_BD_ADDR:

			buf1[0]=0x09;
			buf1[1]=0x10;
			buf1[2]=0;

			Write_HCI_Command( "HCI_CMD_READ_BD_ADDR: " ,3);
			HciState = HCI_CMD_LOCAL_NAME;
			break;

		case HCI_CMD_LOCAL_NAME:

			buf1[0]=0x13;
			buf1[1]=0x0c;
			buf1[2]=0x04;
			buf1[3]='y';
			buf1[4]='t';
			buf1[5]='2';
			buf1[6]=0x00;

			Write_HCI_Command( "HCI_CMD_LOCAL_NAME: " ,7);
			HciState = HCI_CMD_CLASS_DEVICE;
			break;

		case HCI_CMD_CLASS_DEVICE:

			buf1[0]=0x24;
			buf1[1]=0x0c;
			buf1[2]=0x03;
			buf1[3]=0x04;//joystick
			buf1[4]=0x05;//joystick
			buf1[5]=0x00;

			Write_HCI_Command( "HCI_CMD_CLASS_DEVICE: " ,6);
			HciState =HCI_CMD_SCAN_ENABLE;
			break;

		case HCI_CMD_SCAN_ENABLE:

			buf1[0]=0x1a;
			buf1[1]=0x0c;
			buf1[2]=0x01;
			buf1[3]=0x03;

			Write_HCI_Command( "HCI_CMD_SCAN_ENABLE: " ,4);

			//GOTO FIRST CONNECTION
			if(ConnectSwitch==0) HciState = HCI_CMD_WAIT_CONNECTION;
			//GOTO SECOND CONNECTION
			else HciState = HCI_CMD_CREATE_CONNECTION;

			break;

//********************************************************************************
//********************************************************************************
// FIRST CONNECTION STAGE
//********************************************************************************
//********************************************************************************
		case HCI_CMD_WAIT_CONNECTION:

			HciState = HCI_CMD_CONNECTION_ACCEPT;
			break;

		case HCI_CMD_CONNECTION_ACCEPT:

			NVMCON=0x4042;

			TBLPAG=((PROG_ADDR1 & 0x7f0000)>>16);
			addrOffset=(PROG_ADDR1 & 0x00ffff);

			__builtin_tblwtl(addrOffset,0x0000);

			asm("DISI #5");
			__builtin_write_NVM();

			//store BD_address of PC in program memory (memory addresses are PROG_ADDR1 and PROG_ADDR2 )
			NVMCON=0x4003;

			TBLPAG=((PROG_ADDR1 & 0x7f0000)>>16);
			addrOffset=(PROG_ADDR1 & 0x00ffff);

			asm("tblwtl.b %1, [%0]": "=r"(addrOffset) : "d"(buf1[2]));// Write low byte
			addrOffset++;
			asm("tblwtl.b %1, [%0]": "=r"(addrOffset) : "d"(buf1[3]));// Write middle byte
			addrOffset--;
			asm("tblwth.b %1, [%0]": "=r"(addrOffset) : "d"(buf1[4]));// Write high byte
			asm("DISI #5");
			__builtin_write_NVM();

			NVMCON=0x4003;

			TBLPAG=((PROG_ADDR2 & 0x7f0000)>>16);
			addrOffset=(PROG_ADDR2 & 0x00ffff);

			asm("tblwtl.b %1, [%0]": "=r"(addrOffset) : "d"(buf1[5]));// Write low byte
			addrOffset++;
			asm("tblwtl.b %1, [%0]": "=r"(addrOffset) : "d"(buf1[6]));// Write middle byte
			addrOffset--;
			asm("tblwth.b %1, [%0]": "=r"(addrOffset) : "d"(buf1[7]));// Write high byte
			asm("DISI #5");
			__builtin_write_NVM();


			buf1[8]=buf1[7];//copy address
			buf1[7]=buf1[6];
			buf1[6]=buf1[5];
			buf1[5]=buf1[4];
			buf1[4]=buf1[3];
			buf1[3]=buf1[2];
			buf1[0]=0x09;
			buf1[1]=0x04;
			buf1[2]=0x07;
			buf1[9]=0x01;//role

			Write_HCI_Command( "HCI_CMD_CONNECTION_ACCEPT: " ,10);
			HciState =HCI_END;
			break;

//********************************************************************************
//********************************************************************************
		case HCI_CMD_DISCONNECT_REQ:
			buf1[0]=0x06;
			buf1[1]=0x04;
			buf1[2]=0x03;
			buf1[3]=handle[0];
			buf1[4]=handle[1]-0x20;
			buf1[5]=0x13;

			DemoState = BT_STATE_WAIT_DATA;
			Write_HCI_Command( "HCI_CMD_DISCONNECT_REQ: " ,6);
			HciState = HCI_CMD_CONNECTION_ACCEPT;
			break;

//********************************************************************************
//********************************************************************************
// SECOND OR LATER CONNECTION STAGE
//********************************************************************************
//********************************************************************************

		case HCI_CMD_CREATE_CONNECTION:

			//get storde BD_address of PC
			TBLPAG=((PROG_ADDR1 & 0x7f0000)>>16);
			addrOffset=(PROG_ADDR1 & 0x00ffff);

			asm("tblrdl.b [%1], %0": "=r"(buf1[3]) : "r"(addrOffset));// Read low byte
			asm("tblrdl.b [%1], %0": "=r"(buf1[4]) : "r"(addrOffset+1));// Read middle byte
			asm("tblrdh.b [%1], %0": "=r"(buf1[5]) : "r"(addrOffset));// Read high byte

			TBLPAG=((PROG_ADDR2 & 0x7f0000)>>16);
			addrOffset=(PROG_ADDR2 & 0x00ffff);

			asm("tblrdl.b [%1], %0": "=r"(buf1[6]) : "r"(addrOffset));// Read low byte
			asm("tblrdl.b [%1], %0": "=r"(buf1[7]) : "r"(addrOffset+1));// Read middle byte
			asm("tblrdh.b [%1], %0": "=r"(buf1[8]) : "r"(addrOffset));// Read high byte
			HciState = HCI_CMD_CREATE_CONNECTION;

			buf1[0]=0x05;
			buf1[1]=0x04;
			buf1[2]=0x0d;
//			buf1[3]=0x0a;//******************************************
//			buf1[4]=0x6f;//bd_address (6 octets) of the PIC bluetooth
//			buf1[5]=0xfe;//
//			buf1[6]=0x33;//
//			buf1[7]=0x24;//
//			buf1[8]=0x00;//******************************************
			buf1[9]=0x10;
			buf1[10]=0x00;
			buf1[11]=0x00;
			buf1[12]=0x00;
			buf1[13]=0xc2;
			buf1[14]=0xb0;
			buf1[15]=0x00;

			Write_HCI_Command( "HCI_CMD_CONNECTION_COMP: ",16);
			HciState = L2CAP_CON_REQ_TO;
			break;

//********************************************************************************

		case  L2CAP_CON_REQ_TO:
			handle[0]=buf1[3];handle[1]=buf1[4]+0x20;

			buf[0]=handle[0];
			buf[1]=handle[1];
			buf[2]=0x0c;
			buf[3]=0x00;			
			buf[4]=0x08;
			buf[5]=0x00;
			buf[6]=0x01;
			buf[7]=0x00;
			buf[8]=0x02;
			if(hid_flag==0x13) buf[9]=4; else buf[9]=0x01;
			buf[10]=0x04;
			buf[11]=0x00;
			if(hid_flag==0x13) buf[12]=0x13; else buf[12]=0x11;
			buf[13]=0x00;
			if(hid_flag==0x13) buf[14]=0x41; else buf[14]=0x40;
			buf[15]=0x00;

			if(hid_flag!=0x13) hid_flag=0x11;
			acl_busy_flag=1;
		 	Write_ACL_Command("L2CAP_CON_REQ_TO: \r\n      ", 16);
			HciState =  HCI_END;
			break;

//********************************************************************************

		case  HCI_END:
			break;
	}
}


//********************************************************************************
//********************************************************************************
// Dispatcher for L2CAP, SDP, and HID layers
//********************************************************************************
//********************************************************************************

void l2cap_sdp_hid(void)
{
	l2cap_flag=((buf[6]==0x01 && buf[7]==0x00 && buf[8]!=0x07)|| l2cap_pass!=0);
	sdp_flag=((buf[6]>=0x40 && buf[8]==0x06) || sdp_pass!=0);

	//GOTO L2CAP layer****************************
	if(l2cap_flag) l2cap();

	//GOTO SDP layer******************************
	else if(sdp_flag) sdp();

	//GOTO pre_hid function **********************
	else if(hid_flag==0x13) pre_hid();

	//GOTO HID layer******************************
	else if(hid_flag==0xff) hid();

	else return;
}

//********************************************************************************
//********************************************************************************
//L2CAP layer
//********************************************************************************
//********************************************************************************

void l2cap(void){//FIRST CONNECTION

	if(buf[8]==2){

		l2cap_pass=0;

		handle[0]=buf[0];handle[1]=buf[1];
		if(buf[12]==0x11) hid_flag=0x11;
		if(buf[12]==0x13) hid_flag=0x13;
		buf[2]=0x10;
		buf[4]=0x0c;
		buf[8]=0x03;//
		buf[10]=0x08;
		buf[12]=buf[14]+4;//dst_cid (PIC)
		buf[13]=buf[15];//
		buf[16]=0x00;//result
		buf[17]=0x00;//
		buf[18]=0x00;//statis
		buf[19]=0x00;//

		src_cid[0]=buf[14];
		src_cid[1]=buf[15];		

		if(hid_flag==0x11){
			src_cid1[0]=buf[14];
			src_cid1[1]=buf[15];		
		}

		acl_busy_flag=1;
	 	Write_ACL_Command("L2CAP_CONFIG_REQ:\r\n      ", 20);
	}
	else if(buf[8]==3){//SECOND CONNECTION OR LATTER

		l2cap_pass=0;
		src_cid[0]=buf[14];
		src_cid[1]=buf[15];		
		dst_cid[0]=buf[12];
		dst_cid[1]=buf[13];
	}
	else if(buf[8]==4 && ConnectSwitch==0){//FIRST CONNECTION

		if(l2cap_pass!=0) return;
		buf[18]=0x01;//buf[16];//option
		buf[19]=0x02;//buf[17];//
		buf[20]=0x00;//buf[18];//
		buf[21]=0x10;//buf[19];//
		buf[2]=0x12;
		buf[4]=0x0e;
		buf[8]=0x05;//
		buf[10]=0x0a;
		buf[12]=src_cid[0];
		buf[13]=src_cid[1];
		buf[16]=0x00;//result
		buf[17]=0x00;//

 		Write_ACL_Command("L2CAP_CONFIG_RESP:\r\n      ", 22);
		l2cap_pass=1;
	}
	else if(l2cap_pass==1){

		l2cap_pass=0;
		buf[16]=0x01;//buf[18];//option
		buf[17]=0x02;//buf[19];//
		buf[18]=0x00;//buf[20];//
		buf[19]=0x10;//buf[21];//
		buf[2]=0x10;
		buf[4]=0x0c;
		buf[8]=0x04;//
		buf[10]=0x08;
		buf[12]=src_cid[0];
		buf[13]=src_cid[1];
		buf[14]=0x00;//result
		buf[15]=0x00;//

		acl_busy_flag=1;
	 	Write_ACL_Command("L2CAP_CONFIG_REQ_HOST:\r\n      ", 20);

	}
	else if(buf[8]==4 && ConnectSwitch==1){//SECOND OR LATTER CONNECTIONS

		buf[2]=0x0e;
		buf[4]=0x0a;
		buf[8]=0x05;
		buf[10]=0x06;
		buf[12]=dst_cid[0];//=src_cid[0];
		buf[13]=dst_cid[1];//src_cid[1];
		buf[14]=0x00;//result
		buf[15]=0x00;//
		buf[16]=0x00;//
		buf[17]=0x00;//

	 	Write_ACL_Command("L2CAP_CONFIG_REQ_FROM:\r\n      ", 18);
		l2cap_pass=2;
	}
	else if(l2cap_pass==2){

		l2cap_pass=0;
		buf[2]=0x0c;
		buf[4]=0x08;
		buf[8]=0x04;
		buf[9]=0x02;
		buf[10]=0x04;
		buf[12]=dst_cid[0];
		buf[13]=dst_cid[1];
		buf[14]=0x00;//result
		buf[15]=0x00;//
		acl_busy_flag=1;
	 	Write_ACL_Command("L2CAP_CONFIG_REQ_HOST_TO:\r\n      ", 16);
	}
	else if(buf[8]==5 && ConnectSwitch==0){
		acl_busy_flag=1;
	}
	else if(buf[8]==5 && ConnectSwitch==1){
		if(hid_flag==0x11) {hid_flag=0x13;HciState = L2CAP_CON_REQ_TO; hci_acl_init();}
	}
	else if(buf[8]==0x06){
		buf[8]=0x07;
		disconnection_flag++;
	if(hid_flag!=0x13) acl_busy_flag=1;
	 	Write_ACL_Command("L2CAP_DISCONNECTION:\r\n      ", 16);
	}

	else return;
}

//********************************************************************************
//********************************************************************************
//SDP layer
//********************************************************************************
//********************************************************************************
void sdp(void){

	if((buf[16]==0x01 && buf[27]==0) || sdp_pass==1) {
		if(step==0){
			for(step=0;step<5;step++){
			 	att1a[step][0]=buf[0];
				att1b[step][0]=buf[0];
			}
			att1c[0][0]=buf[0];
			att1c[1][0]=buf[0];

			//copy src_cid to SDP data		
			att1a[0][6]=src_cid[0];	att1a[0][7]=src_cid[1];
			att1b[0][6]=src_cid[0];	att1b[0][7]=src_cid[1];
			att1c[0][6]=src_cid[0];	att1c[0][7]=src_cid[1];

			step=0;
			//copy ID to SDP data
			att1a[0][9]=buf[9];att1a[0][10]=buf[10];
			sdp_pass=1;
		}
		if(step!=5){
			data_size=att1a[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) buf[pdata]=att1a[step][pdata];
			step++; 
			if(step==5) {step=0;sdp_pass=0;acl_busy_flag=1;}
			Write_ACL_Command("SDP1: ", data_size);
		}
	}

	else if((buf[16]==0x01 && buf[27]==0x02 && buf[28]==0x00 && buf[29]==0x76) || sdp_pass==2) {
		if(step==0){
			//copy ID to SDP data
			att1b[0][9]=buf[9];att1b[0][10]=buf[10];
			sdp_pass=2;
		}
		if(step!=5){
			data_size=att1b[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att1b[step][pdata];
			step++; 
			if(step==5) {step=0;sdp_pass=0;acl_busy_flag=1;}
			Write_ACL_Command("SDP2: ", data_size);
		}
	}

	else if((buf[16]==0x01 && buf[27]==0x02 && buf[28]==0x00 && buf[29]==0xEC) || sdp_pass==3) {
		if(step==0){
			//copy ID to SDP data
			att1c[0][9]=buf[9];att1c[0][10]=buf[10];
			sdp_pass=3;
		}
		if(step!=2){
			data_size=att1c[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att1c[step][pdata];
			step++; 
			if(step==2) {step=0;sdp_pass=0;acl_busy_flag=1;}
			Write_ACL_Command("SDP3: ", data_size);
		}
	}

	else if(buf[16]==0x12 || sdp_pass==4) {
		if(step==0){

			//copy handle to SDP data
			for(step=0;step<4;step++){
			 	att2[step][0]=buf[0];
			}

			step=0;

			//copy src_cid to SDP data		
			att2[0][6]=src_cid[0];	att2[0][7]=src_cid[1];

			//copy ID to SDP data
			att2[0][9]=buf[9];att2[0][10]=buf[10];
			sdp_pass=4;
		}

		if(step!=4) {
			data_size=att2[step][2]+4;
			for(pdata=0;pdata<data_size;pdata++) 	buf[pdata]=att2[step][pdata];
			step++; 
			if(step==4) {step=0;sdp_pass=0;acl_busy_flag=1;}
			Write_ACL_Command("SDP4: ", data_size);
		}
	}
	else return;
}


//********************************************************************************
//********************************************************************************
//pre_hid function
//********************************************************************************
//********************************************************************************

void pre_hid(void){

// ACL: 2A 20 06 00 02 00 45 00 90 00
	if(buf[8]==0x90) {hid_pass=1;return;}

//UART2PrintString( "written1 *********************** \r\n");
	if(disconnection_flag==0 && ConnectSwitch==0) return;
//UART2PrintString( "written2 *********************** \r\n");

	if(hid_pass==1){
		hid_flag=0xff;
		hid_pass=2;
		buf1[0]=0x1a;
		buf1[1]=0x0c;
		buf1[2]=0x01;
		buf1[3]=0x00;
		Write_HCI_Command( "HCI_CMD_SCAN_DISABLE: " ,4);
	}
}


//********************************************************************************
//********************************************************************************
//HID layer
//********************************************************************************
//********************************************************************************
void hid(void){
	if(hid_pass==2){

			/*
				set characters (USB HID codes: see an attached file "translate.pdf")
				in buffers (from buf[9] to buf[16])
					
				set following modiferkey in buf[9]
				ModiferKeyTable
					0x01 ; L CTRL
					0x02 ; L SHFT
					0x04 ; L ALT
					0x08 ; L GUI
					0x10 ; R CTRL
					0x20 ; R SHFT
					0x40 ; R ALT
					0x80 ; R GUI
			*/

			do{// to check a key-push
			key_data[0]=PORTAbits.RA0;
			key_data[1]=PORTAbits.RA1;
			key_data[2]=PORTBbits.RB2;
			key_data[3]=PORTBbits.RB3;
			}
			while(3!=key_data[0]+ key_data[1]+ key_data[2]+ key_data[3]);

			//to notify the key-push to PC
			if(key_data[0]==0x00) {buf[9]=0x00;buf[11]=4;}//HID Usage ID 'a' 
			else if(key_data[1]==0x00) {buf[9]=0x00;buf[11]=5;}//HID Usage ID 'b'
			else if(key_data[2]==0x00) {buf[9]=0x02;buf[11]=4;}//HID Usage ID 'A'
			else if(key_data[3]==0x00) {buf[9]=0x02;buf[11]=5;}//HID Usage ID 'B'

			buf[0]=handle[0];
			buf[1]=handle[1];
			buf[2]=0x0d;
			buf[3]=0x00;
			buf[4]=0x09;
			buf[5]=0x00;
			buf[6]=src_cid[0];//0x41;
			buf[7]=src_cid[1];//0x00;
			buf[8]=0xa1;
//			buf[9]=0;
			buf[10]= 0;
//			buf[11]= 0;
			buf[12]= 0;
			buf[13]= 0;
			buf[14]= 0;
			buf[15]= 0;
			buf[16]= 0;
			data_size=17;

			hid_pass=3;
			Write_ACL_Command("A key is pushed: ", data_size);
		    return;
	}
	else{

			buf[0]=handle[0];
			buf[1]=handle[1];
			buf[2]=0x0d;
			buf[3]=0x00;
			buf[4]=0x09;
			buf[5]=0x00;
			buf[6]=src_cid[0];//0x41;
			buf[7]=src_cid[1];//0x00;
			buf[8]=0xa1;
			buf[9]=0;
			buf[10]= 0;
			buf[11]= 0;
			buf[12]= 0;
			buf[13]= 0;
			buf[14]= 0;
			buf[15]= 0;
			buf[16]= 0;
			data_size=17;

   			DelayMs(50); // 50ms delay
			do{//to check the key-release
			key_data[0]=PORTAbits.RA0;
			key_data[1]=PORTAbits.RA1;
			key_data[2]=PORTBbits.RB2;
			key_data[3]=PORTBbits.RB3;
			}
			while(4!=key_data[0]+ key_data[1]+ key_data[2]+ key_data[3]);

			hid_pass=2;
			Write_ACL_Command("No key is pressed: ", data_size);
		    return;
	}
}

//********************************************************************************
//********************************************************************************
// WRITE functions
//********************************************************************************
//********************************************************************************

void Write_ACL_Command(char *message, int data_size2)
{

	hci_busy_flag=1; buf1[0]=0xff;

	#ifdef DEBUG_MODE
		UART2PrintString( message);
		for(data_num=0;data_num<data_size2;data_num++)
	      	 {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
	#endif	

    if (!USBHostGenericTxIsBusy(deviceAddress))
	{
		USBHostGenericAclWrite( deviceAddress, buf, data_size2 );
	}
	else{
		UART2PrintString( "ACL cannot be written *********************** \r\n");
	}
}

void Write_HCI_Command(char *message, int data_size1)
{
	#ifdef DEBUG_MODE
		UART2PrintString( message);
		for(data_num=0;data_num<data_size1;data_num++)
	      	 {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
	#endif	


	if ( USBHostGenericClassRequest( deviceAddress, buf1, data_size1 ) == USB_SUCCESS )
	{
//	UART2PrintString( "HCI COMMAND *\r\n" );
	}
	else
	{
	UART2PrintString( "Write Class Error !\r\n" );	
	}
}

/*************************************************************************
 * Function:        ManageDemoState
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *************************************************************************/
void ManageDemoState ( void )
{

    // Watch for device detaching
    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
        UART2PrintString( "Generic demo device detached - polled\r\n" );
        DemoState = BT_INITIALIZE;
        deviceAddress   = 0;
    }

    switch (DemoState)
    {
		case BT_INITIALIZE:
			DemoState = BT_STATE_IDLE;
        	break;

   	 /** Idle State:  Loops here until attach **/
	    case BT_STATE_IDLE:
      	  if (CheckForNewAttach())
      	  {
				DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET; HidState= HID_START;
      	  }
      	  break;

  	  case BT_STATE_PROCESS:

			buf1[0]=0x03;
			buf1[1]=0x0c;
			buf1[2]=0;

			Write_HCI_Command( "HCI_CMD_RESET: " ,3);

			DemoState = BT_STATE_WAIT_DATA;
			break;

    	case BT_STATE_WAIT_DATA:
			if (!USBHostGenericRx1IsBusy(deviceAddress) ) {
				USBHostGenericRead(deviceAddress,buf1,DATA_PACKET_LENGTH);
			}
			else{
				UART2PrintString( "HCI cannot be armed \r\n" );
			}
			if (!USBHostGenericRx2IsBusy(deviceAddress)) {
				USBHostGenericAclRead(deviceAddress, buf, DATA_PACKET_LENGTH);
			}
			break;
	}
    DelayMs(1); // 1ms delay

} // ManageDemoState


//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        USB_ApplicationEventHandler
 *
 * Preconditions:   The USB must be initialized.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          deviceAddress (global)
 *                  Updates device address when an attach or detach occurs.
 *
 *                  DemoState (global)
 *                  Updates the demo state as appropriate when events occur.
 *
 * Returns:         TRUE if the event was handled, FALSE if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer or client
 *                  driver to notify the application of events that occur.
 *                  If the event is recognized, it is handled and the
 *                  routine returns TRUE.  Otherwise, it is ignored (or
 *                  just "sniffed" and the routine returns FALSE.
 *************************************************************************/

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
        BYTE i;
    #endif

    // Handle specific events.
    switch ( (INT)event )
    {
        case EVENT_GENERIC_ATTACH:
            if (size == sizeof(GENERIC_DEVICE_ID))
            {
                deviceAddress   = ((GENERIC_DEVICE_ID *)data)->deviceAddress;
                DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET; HidState= HID_START;//YTS !!!!!!!!!!!!!!!!!
                UART2PrintString( "Generic demo device attached - event, deviceAddress=" );
                UART2PutDec( deviceAddress );
                UART2PrintString( "\r\n" );
                #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
                    for (i=1; i<((GENERIC_DEVICE_ID *)data)->serialNumberLength; i++)
                    {
                        UART2PutChar( ((GENERIC_DEVICE_ID *)data)->serialNumber[i] );
                    }
                #endif
                UART2PrintString( "\r\n" );
                return TRUE;
            }
            break;

        case EVENT_GENERIC_DETACH:
            deviceAddress   = 0;
            DemoState = BT_INITIALIZE;//YTS
            UART2PrintString( "Generic demo device detached - event\r\n" );
            return TRUE;

        case EVENT_GENERIC_TX_DONE:           // The main state machine will poll the driver.
            return TRUE;

        case EVENT_GENERIC_RX1_DONE:

			if(*(int*)data != 0){

				#ifdef DEBUG_MODE
				UART2PrintString( " HCI: " );
				for(data_num=0;data_num<*(int*)data;data_num++)
	      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
				UART2PrintString( "\r\n" );
				#endif
				if(buf1[0]==0x05) {
					HciState=HCI_CMD_DISCONNECT_REQ;
					hci_acl_init();
				}
				else if(buf1[0]==0x03 && buf1[2]!=0) {
					HciState=HCI_CMD_CREATE_CONNECTION;
					hci_acl_init();
				}
				else if(buf1[0]==0x0e && hid_flag==0xff) {
					UART2PrintString( "hid \r\n" );
					LATBbits.LATB15=1;
					hid();
				}
				else if(buf1[0]==0x0e || buf1[0]==0x03|| buf1[0]==0x04) hci_acl_init();
				else if(buf1[0]==0x13 && hci_busy_flag==1) {
					hci_busy_flag=0;
					if(acl_busy_flag==0) l2cap_sdp_hid();
				}
			}
            return TRUE;

        case EVENT_GENERIC_RX2_DONE:

			#ifdef DEBUG_MODE
			UART2PrintString( " ACL: " );
			for(data_num=0;data_num<*(DWORD*)data;data_num++)
				{UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			acl_busy_flag=0;

			if(hci_busy_flag==0)l2cap_sdp_hid();
            return TRUE;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return TRUE;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return TRUE;

        case EVENT_HUB_ATTACH:
            UART2PrintString( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            UART2PrintString( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            UART2PrintString( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            UART2PrintString( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            UART2PrintString( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            UART2PrintString( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return TRUE;
            break;

        default:
            break;
    }

    return FALSE;

} // USB_ApplicationEventHandler


//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{
    // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - system.  Halting.\r\n\r\n" );
        while (1);
    }
    if ( USBHostInit(0) == TRUE )
    {
        UART2PrintString( "\r\n\r\n***** USB Custom Demo App Initialized *****\r\n\r\n" );
    }
    else
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - USB.  Halting.\r\n\r\n" );
        while (1);
    }

    // Main Processing Loop
    while (1)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBHostTasks();
        // Maintain Demo Application State
        ManageDemoState();
    }

    return 0;

} // main


/*************************************************************************
 * EOF main.c
 */

